# Copyright (C) 2021-2022 by Ivan.
# This file is part of Snowflake package.
# Snowflake is released under the MIT License (see LICENSE).


from .snowflake import *


def version():
    return "0.0.3"
